/*
    BCal - Blacky Calendar

    Copyright (C) Andreas Hofmeier

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License as
    published by the Free Software Foundation; either version 2 of
    the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
    General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


    Name:     Andreas Hofmeier

    Adresse:  Stralsunder Strae 20
              28239 Bremen
              Germany

    Telefon:  +49-421-613793

    eMail:    An-H@An-H.de
              hofmeier@inbremen.de
              an-h@gmx.de

    Homepage: http://www.An-H.de/
	      hofmeier.home.pages.de
              http://www.LGut.uni-bremen.de/anh/

*/


#include <signal.h>
#include <stdio.h>
#include <unistd.h>
#include <time.h>

#include <sys/time.h>
#include <sys/types.h>
#include <sys/stat.h>

#include <math.h>

#include <ncurses.h>
#include "hofmeier.h"

char *EDITOR;
char *EDITOR2;
char *VIEWER;

char blacky_ver_info[] = "Blacky Calendar v0.30 (c) Copyright 2000-2004 Andreas Hofmeier";


/* function zum Herunterfahren ! */
static void finish(int sig);
/* UHR function */
static void uhr(int sig);
/* Windowsgre gendert */
static void winversch(int sig);

char *makefilenamefromdate(unsigned long date);
unsigned long getaktdate(int d, int w);



/* Lnge der normalen "Mll-Arrays" */
#define MULLLEN /*512*/ 256

// Zeit jetzt
unsigned long timedatenow;
// Temp-Sting fr Datumsausgabe
char timedatenow_timestr[15], timedatenow_datestr[200];
/* Zeit Datum */
int year, mon, day, dow, stu, min, week, doy, sec;
/* Tage als String */
char *tage[] = { "So", "Mo", "Di", "Mi", "Do", "Fr", "Sa" };
char *tage_ganz[] = { "Sonntag", "Montag", "Dienstag", "Mittwoch", "Donnerstag", "Freitag", "Samstag" };
/* Monate als 3Bytes String */
char *mon3[] = { "Jan", "Feb", "Mar", "Apr", "Mai", "Jun", "Jul", "Aug", "Sep", "Okt", "Nov", "Dez"};
char *mona[] = {"Januar", "Februar", "Mrz", "April", "Mai", "Juni", "Juli", "August", "September", "Oktober", "November", "Dezember"};
/* Anzahl der Tage enes Monates */
char mont[]  = {  31,   30,    31,    30,    31,    30,    31,    31,    30,     31,   30,    31};
// Mu Datum angepasst werden?
int xdow = 100, xday = 255;
// Mu Kalendar-Display neu berechnet werden?
unsigned long xdateanz;
// Datum: Mittelpunkt Anzeige
unsigned long dateanz;
// Wiefiele Secunden hat ein Tag
#define SECOUNDADAY 24*60*60
// Zwischenspeicherung des Canlendars
#define CALDIS_WEEKS 20
#define CALDIS_WEEKSSPACE 4
unsigned char caldis[10][CALDIS_WEEKS + CALDIS_WEEKSSPACE + 1]; // Hm, wenn hier +1 steht, wird ein hufiger auftretendes Sem.Faut vermieden
// Cursor-Possition in der Matrix
int curpos_w = 0, curpos_d = 0;
// Possition des Mittelpunktes
int mpos_w = 0, mpos_d = 0;
// search-result-file 
FILE *srf;
char searchstring[256];
// TimePoint 0
unsigned long tp0;

// Tage in Array ("Zeilen" im Buffer)
#define LINEBUFFERSIZE 18
// Bildschirmzeilen
#define LINESIZE 18
// Maximale Anzahl der Eintrge/Termine TPs/DL/TMs pro Tag
#define LINEBUFFERMAXPERDAY 24

// Buffer fr den Zeilenmode
struct LINEBUFFER {
  int dow;
  int day;
  int mon;
  int year;
  int week;
  int attr; // Farbe, etc
  char *note[LINEBUFFERMAXPERDAY]; // = NULL;
} linebuffer[LINEBUFFERSIZE];
// Possition des Cursurs im Line-Mode
int line_cursor = 0;

int linedisplay(int mode, unsigned long date_anfw);
linefillbuffer(int mode, unsigned long date_anf);
unsigned long linegetaktdate(int line);
unsigned long linebgetaktdate(int bline);


// Die History fr die Back-Funktion
int histsize = 10;
int inhist = 0;
unsigned long hist[10];

// Verarbeiten der Blacky2000-Timer-Datei
char *Blacky2000TMR = NULL;
char *run_timer();
// Welche Timer-Gruppen in der Blacky2000TMR-File sollen ein Highlight bewirken?
char *Blacky2000TMR_HL = NULL;
// HightLight ya, wenn == 1
int btmrgrpok;


// Wo werden die Notice-Files hingeschrieben
char *blktxt;

pss(s, s2)
  char *s, s2[];
{
  if (s == NULL)
    finish(-1);
}


/* Kopiert Zeit und Datum in die Globalen Variablen */
gettimedate()
{
  //  long ti;
  static struct tm *tminfo;

  time(&timedatenow);
  tminfo = localtime(&timedatenow);
  year = (tminfo -> tm_year) + 1900; /* Jahr        */
  mon = (tminfo -> tm_mon) + 1;      /* Monat       */
  day = (tminfo -> tm_mday);         /* Tag         */
  dow = (tminfo -> tm_wday);         /* Wochentag   */
  stu = (tminfo -> tm_hour);         /* Stunde      */
  min = (tminfo -> tm_min);          /* Minute      */
  sec = (tminfo -> tm_sec);          /* Sekunde     */
  doy = (tminfo -> tm_yday);         /* Tag im Jahr */
  /* Woche */
  if (xday == day) {
    return;
  } else {
    week = getweek(timedatenow);
    xday = day;
  }
}


/* Gibt eine Kurtzhilfe im unteren Fenster-Bereich aus
   mod = 0: without cleaning
         1: with cleaning */
painthelp(mod)
{
  if (mod) {
    makeline(RABGE, HLINE, LABGE, 1, 15);
    makeline(VLINE, NOMBG, VLINE, 1, 16);
    makeline(VLINE, NOMBG, VLINE, 1, 17);
    makeline(VLINE, NOMBG, VLINE, 1, 18);
    makeline(VLINE, NOMBG, VLINE, 1, 19);
    makeline(VLINE, NOMBG, VLINE, 1, 20);
    makeline(VLINE, NOMBG, VLINE, 1, 21);
    makeline(VLINE, NOMBG, VLINE, 1, 22);
    makeline(VLINE, NOMBG, VLINE, 1, 23);
    makeline(LUECK, HLINE, RUECK, 1, 24);
  }

  gotoxy(3, 17);
  addstr("t=go_ToDay g=goto ENTER,_=Edit;v=View_Notice <-,->,pg(up,dn)=Select B=Back");
  gotoxy(3, 18);
  addstr("/,n=Search,next c=calculate_past_time b=B2000TMR ?=this_help_screen x=Exit");
}


/* ndert die Attributs-Bits auf grund des Files oder des timer-files
   bergeben wird das Aktuelle Datum (Anzeige) und die Position in der
   Martix (curpos_d, curpos_w)
*/
calcattr(d, w) {
  FILE *f;
  char line[64];
  unsigned char add = 0;
  int i;

  if ((caldis[d][w] & 224) / (2 * 2 * 2 * 2 * 2) == 1) {
    return;
  }

  if ((f = fopen(makefilenamefromdate(getaktdate(d, w)), "r")) != NULL) {
    line[0] = 0;
    fgets(line, 63, f);
    if (line[0] != 0) {
      add = 2;
    }
    if (line[0] == '=') {
      Gross(line, line);
      if (strncmp(line + 1, "GREEN", 5) == 0) {
	add = 3;
      } else {
	if (strncmp(line + 1, "RED", 3) == 0) {
	  add = 4;
	} else {
	  if (strncmp(line + 1, "MAGENTA", 7) == 0) {
	    add = 5;
	  } else {
	    if (strncmp(line + 1, "BLUE", 4) == 0) {
	      add = 6;
	    } else {
	      if (strncmp(line + 1, "YELLOW", 6) == 0) {
		add = 7;
	      }
	    }
	  }
	}
      }
    }
    fclose(f);
  }
  caldis[d][w] = (unsigned char) ((caldis[d][w] & 31) + add * (2 * 2 * 2 * 2 * 2));
}


/* mod = 0: Mormal
       = 1: Mit ZeigeInfo
       = 2: Linemode */
paint(mod)
  char mod;
{
  static int i, i2, j, k, m;
  static struct tm *tminfo, tmdata;
  static unsigned long daydate_anf = 0, tmptime;
  static char ausg[150];
  static struct tm *timeptr;

  static FILE *f;

  uhr(128);

  if (dateanz == 0) {
    gettimedate();
    dateanz = timedatenow;
  }

  makeline(LOECK, HLINE, ROECK, 1, 1);
  makeline(VLINE, NOMBG, VLINE, 1, 2);
  // Verinfo
  gotoxy(3, 2);
  addstr(blacky_ver_info);
  makeline(RABGE, HLINE, LABGE, 1, 3);
  makeline(VLINE, NOMBG, VLINE, 1, 4);
  // TIME
  gotoxy(3, 4);
  addstr(timedatenow_timestr);
  // Date
  gotoxy(12, 4);
  addstr(timedatenow_datestr);

  makeline(RABGE, HLINE, LABGE, 1, 5);

  //#ifdef BLACKY_cur_Graphik
  //  attrset(A_ALTCHARSET | A_NORMAL & ~A_BLINK);
  //  attrset(A_ALTCHARSET);
  //#endif
  //  gotoxy(47, 3);
  //  addch(OABGE);
  makepoint(OABGE, 47, 3);

  //  gotoxy(47, 4);
  //  addch(VLINE);
  makepoint(VLINE, 47, 4);

  //  gotoxy(47, 5);
  //  addch(RUECK);
  makepoint(RUECK, 47,5);

  addstr("                                ");

  //  addch(VLINE);
  makepoint(VLINE, 80, 5);
  //#ifdef BLACKY_cur_Graphik
  //  attrset(A_NORMAL);
  //#endif
  makeline(VLINE, NOMBG, VLINE, 1, 6);
  makeline(VLINE, NOMBG, VLINE, 1, 7);
  makeline(VLINE, NOMBG, VLINE, 1, 8);
  makeline(VLINE, NOMBG, VLINE, 1, 9);
  makeline(VLINE, NOMBG, VLINE, 1, 10);
  makeline(VLINE, NOMBG, VLINE, 1, 11);
  makeline(VLINE, NOMBG, VLINE, 1, 12);
  makeline(VLINE, NOMBG, VLINE, 1, 13);
  makeline(VLINE, NOMBG, VLINE, 1, 14);

  if (mod == 2) {
    makeline(VLINE, NOMBG, VLINE, 1, 15);
  } else {
    makeline(RABGE, HLINE, LABGE, 1, 15);
  }
  makeline(VLINE, NOMBG, VLINE, 1, 16);
  makeline(VLINE, NOMBG, VLINE, 1, 17);
  makeline(VLINE, NOMBG, VLINE, 1, 18);
  makeline(VLINE, NOMBG, VLINE, 1, 19);
  makeline(VLINE, NOMBG, VLINE, 1, 20);
  makeline(VLINE, NOMBG, VLINE, 1, 21);
  makeline(VLINE, NOMBG, VLINE, 1, 22);
  makeline(VLINE, NOMBG, VLINE, 1, 23);
  makeline(LUECK, HLINE, RUECK, 1, 24);

  if (mod < 2) {
    // Calendar
    // nur neu berechnen, wenn es wirklich notwendig ist
    if ((long) dateanz != (long) xdateanz) {
      int xxmon = 255;
      xdateanz = dateanz;
      //exit(0);
      
      tminfo = localtime(&dateanz);
      
      memset((unsigned char *) caldis, 0, (CALDIS_WEEKS + CALDIS_WEEKSSPACE) * 9);
      // Welcher Tag ist der erster in der Anzeige?
      daydate_anf = dateanz - ((((tminfo -> tm_wday == 0) ? 6 : (tminfo -> tm_wday) - 1) + ((CALDIS_WEEKS / 2 - 1) * 7)) * SECOUNDADAY);
      for (i = 0; i < CALDIS_WEEKS + CALDIS_WEEKSSPACE; i++) {
	for (j = 0; j < 7; j++) {
	  tminfo = localtime(&daydate_anf);
	  
	  if ((int) xxmon != (int) (tminfo -> tm_mon)) {
	    // Die erste Reihe, sollte nur einen Monat enthalten
	    if ((xxmon != 255) && ((tminfo -> tm_mday) == 1) && (i == 0)) {
	      int x;
	      
	      for (x = 0; x < j; x++) {
		caldis[x][i] = 0;
	      }
	    }
	    xxmon = (int) (tminfo -> tm_mon);
	    if (i != 0) {
	      // Beugt mll zwischen zwei monaten vor...
	      if ((caldis[0][i] == 0) && (caldis[7][i] == 0)) {
		caldis[8][i] = 0;
	      }
	      i++;
	    }
	  }
	  
	  caldis[j][i] = (tminfo -> tm_mday);
	  
	  if (j == 0) {
	    caldis[7][i] = getweek(daydate_anf);
	  }
	  
	  //	if (j == 6) {
	  caldis[8][i] = tminfo -> tm_mon + 1;
	  caldis[9][i] = tminfo -> tm_year;
	  //	}
	  
	  
	  if (daydate_anf == dateanz) {
	    // Cursor (und Mittelpunktzeiger) auf Anzeigemittelpunkt
	    mpos_w = curpos_w = i;
	    mpos_d = curpos_d = j;
	  }
	  
	  // Aktueller Tag?
	  if (year == (tminfo -> tm_year + 1900)) {
	    if (mon == (tminfo -> tm_mon + 1)) {
	      if (day == (tminfo -> tm_mday)) {
		caldis[j][i] += 1 * (2 * 2 * 2 * 2 * 2); 
	      }
	    }
	  }
	  
	  // Notiz? Attribute von dieser?
	  calcattr(j,i);
	  
	  daydate_anf += SECOUNDADAY;
	}
      }
    }
    // ... und anzeigen
    // Monate, ganz oben
    for (j = i = 0; i < CALDIS_WEEKS + CALDIS_WEEKSSPACE; i++) {
      if (caldis[8][i] != 0) {
	if (j != caldis[8][i]) {
	  j = caldis[8][i];
	  gotoxy((k = (6 -1 + (i * 3))), 6);
	  //	addstr(mona[j - 1]);
	  sprintf(ausg, " %s (%02d)", mona[j - 1], j);
	  //             ^ -1 !
	  if ((k + (m = strlen(ausg))) > 78 /*80Zeichen Breiter Terminal*/) {
	    // Hm, es sieht niocht gerade gut aus, wenn die Zeilen zu lang sind.
	    ausg[(m - ((k + m) - 78)) + 1] = 0;
	  }
	  addstr(ausg);
	}
      }
    }
    
    // Woche
    attrset(A_STANDOUT);
    // Ecke links oben ausfllen
    gotoxy(3, 7);
    addstr("  ");
    gotoxy(5, 7);
    for (i = 0; i < CALDIS_WEEKS + CALDIS_WEEKSSPACE; i++) {
      if (caldis[7][i] != 0) {
	if (curpos_w != i) {
	  printw(" %02d", caldis[7][i]);
	} else {
	  addstr(" ");
	  attrset(A_NORMAL);
	  printw("%02d", caldis[7][i]);
	  attrset(A_STANDOUT);
	}
      } else {
	addstr("   ");
      }
    }
    attrset(A_NORMAL);
    
    // Wochentage in Monaten
    for (j = 0; j < 7; j++) {
      // Wochentag
      gotoxy(3, 8 + j);
      if (curpos_d != j) {
	attrset(A_STANDOUT);
      }
      addstr(tage[(j == 6) ? 0 : j + 1]);
      attrset(A_NORMAL);
      
      for (i = 0; i < CALDIS_WEEKS + CALDIS_WEEKSSPACE; i++) {
	if (caldis[j][i] != 0) {
	  if ((curpos_w == i) && (curpos_d == j)) {
	    addstr(" ");
	    attrset(A_STANDOUT);
	    printw("%02d", (caldis[j][i] & 31));
	    attrset(A_NORMAL);
	  } else {
	    addstr(" ");
	    // Hochwertigen drei Bits fr Attribute
	    switch((caldis[j][i] & 224) / (2 * 2 * 2 * 2 * 2)) {
	      // Normal - keine Hervorhebung
	    case 0:
	      break;
	      // Blink - Heute
	    case 1:
	      attrset(A_BLINK);
	      break;
	      // Normale Hervorhebung - White/Underline
	    case 2:
	      attrset(A_UNDERLINE);
	      break;
	      // Green
	    case 3:
	      attrset(COLOR_PAIR(COLOR_GREEN)|A_BOLD);
	      break;
	      // RED
	    case 4:
	      attrset(COLOR_PAIR(COLOR_RED)|A_BOLD);
	      break;
	      // Lila
	    case 5:
	      attrset(COLOR_PAIR(COLOR_MAGENTA)|A_BOLD);
	      break;
	      // Blau
	    case 6:
	      attrset(COLOR_PAIR(COLOR_BLUE)|A_BOLD);
	      break;
	    case 7:
	      // Gelb
	      attrset(COLOR_PAIR(COLOR_YELLOW)|A_BOLD);
	      break;
	    } /* switch */
	    printw("%02d", (caldis[j][i] & 31));
	    attrset(A_NORMAL);
	  }
	} else {
	  addstr("   ");
	}
      }
    }



    // Blacky2000 TMR File
    i = 0; // Laufvariable fr Info-Feld...
    if (Blacky2000TMR != NULL) {
      static char *ret;
      ret = run_timer(0, getaktdate(curpos_d, curpos_w));
      if ((ret != NULL) && (ret[0] != 0)) {
	gotoxy(3, 16 + i);
	addstr(ret);
	i++;
      }
    }
    // Info-Datei
    if (((caldis[curpos_d][curpos_w] & 224) / (2 * 2 * 2 * 2 * 2)) > 0) {
      if ((f = fopen(makefilenamefromdate(getaktdate(curpos_d, curpos_w)), "r")) != NULL) {
	dosread(f, ausg, 150);
	if (ausg[0] != '=') {
	  gotoxy(3, 16 + i);
	  addstr(ausg);
	  i++;
	}
	for (/*i = 0*/; ((i < 8) && (!feof(f))); i++) {
	  dosread(f, ausg, 150);
	  gotoxy(3, 16 + i);
	  addstr(ausg);
	}
	fclose(f);
      }
    } else {
      painthelp(0);
    }
    
    // Ausgewhltes Datum
    gotoxy(49, 4);
    tmptime = getaktdate(curpos_d, curpos_w);
    timeptr = (struct tm *) localtime((const time_t *) &tmptime);
    memcpy((char *) &tmdata, (char *) timeptr, sizeof(tmdata));
    timeptr = &tmdata;
    
    if (((caldis[curpos_d][curpos_w] & 224) / (2 * 2 * 2 * 2 * 2)) == 1) {
      addstr("*********** TODAY ************");
    } else {
      printw("%02s, %02d/%02d. %03s(%02d) %04d %03d/%02d",
	     tage[timeptr -> tm_wday],
	     timeptr -> tm_mday,
	     mont[timeptr -> tm_mon],
	     mon3[timeptr -> tm_mon],
	     (timeptr -> tm_mon) + 1,
	     (timeptr -> tm_year) + 1900,
	     (timeptr -> tm_yday) + 1,
	     getweek(tmptime));
    }
    
    // ENDE Back Funktion
    if (inhist != 0) {
      if (tmptime != hist[inhist]) {
	inhist = 0;
      }
    }
    // save history
    if (tmptime != hist[inhist]) {
      for (i = histsize - 1; i > 0; i--) {
	hist[i] = hist[i - 1];
      }
      hist[0] = tmptime;
    }
  }

  if (mod == 2) {
    int wakt = 99; // Nicht immer die gleiche Wochenzahl untereinander. 

    for (j = i = 0; j < LINESIZE; i++) {
      if (linebuffer[i].day == -1) {
	break;
      }

      // Hochwertigen drei Bits fr Attribute
      switch(linebuffer[i].attr) {
	// Normal - keine Hervorhebung
      case 0:
	break;
	// Blink - Heute
      case 1:
	attrset(A_BLINK);
	break;
	// Normale Hervorhebung - White/Underline
      case 2:
	attrset(A_UNDERLINE);
	break;
	// Green
      case 3:
	attrset(COLOR_PAIR(COLOR_GREEN)|A_BOLD);
	break;
	// RED
      case 4:
	attrset(COLOR_PAIR(COLOR_RED)|A_BOLD);
	break;
	// Lila
      case 5:
	attrset(COLOR_PAIR(COLOR_MAGENTA)|A_BOLD);
	break;
	// Blau
      case 6:
	attrset(COLOR_PAIR(COLOR_BLUE)|A_BOLD);
	break;
      case 7:
	// Gelb
	attrset(COLOR_PAIR(COLOR_YELLOW)|A_BOLD);
	break;
      } /* switch */
    
      if (wakt != linebuffer[i].week) {
	wakt = linebuffer[i].week;
	sprintf(ausg, "%02d %s %02d.%02d.%04d ", linebuffer[i].week, tage[linebuffer[i].dow],
		linebuffer[i].day, linebuffer[i].mon, linebuffer[i].year);
      
      } else {
	sprintf(ausg, "   %s %02d.%02d.%04d ", tage[linebuffer[i].dow],
		linebuffer[i].day, linebuffer[i].mon, linebuffer[i].year);
      }

      gotoxy(3, 6 + j);
      addstr(ausg);

      attrset(A_NORMAL);

      for (i2 = 0; i2 < LINEBUFFERMAXPERDAY; i2++) {
	if (j >= LINESIZE) {
	  break;
	}
	if (linebuffer[i].note[i2] != NULL) {
	  if (line_cursor == j) {
	    attrset(A_STANDOUT);
	  }
	  gotoxy(21, 6 + j++);
	  
	  addstr(linebuffer[i].note[i2]);
	  for (k = 58 - strlen(linebuffer[i].note[i2]); k > 0; k--) {
	    addstr(" ");
	  }
	  attrset(A_NORMAL);
	}
      }
      if (linebuffer[i].note[0] == NULL) {
	if (line_cursor == j) {
	  attrset(A_STANDOUT);
	
	  gotoxy(21, 6 + j);
	  // 58 Zeichen
	  addstr("                                                          ");
	  attrset(A_NORMAL);
	}
	j++;
      }
    }
  }
  
  // kein Cursor
  curs_set(0);
    
  refresh(); 
}


char *makefilenamefromdate(unsigned long date)
{
  static char cd[256];
  static struct tm *timeptr;

  timeptr = localtime(&date);
  sprintf(cd, "%s/bcal%04d%02d%02d.txt", blktxt, timeptr -> tm_year + 1900, (timeptr -> tm_mon + 1), timeptr -> tm_mday);
  return cd;
}


unsigned long getaktdate(int d, int w)
{
  static struct tm timeptr;

  memset(&timeptr, 0, sizeof(timeptr));

  timeptr.tm_mday = (int) (caldis[d][w] & 31);
  timeptr.tm_mon  = (int) (caldis[8][w] - 1);
  timeptr.tm_year = (int) (caldis[9][w]);

  // Schaltsekunden: Von Zeit zu Zeit treten Schaltsekunden auf. Ist
  // die Zeit dann 0:00:00 wird nach 23:59:59 umgeschaltet, und das
  // ist der vorherige Tag. Dies frd zu Fehlern. Ist die Zeit
  // 12:00:00, ist die Schwelle weit entfernt...
  timeptr.tm_hour = (int) 12;

  return (unsigned long) mktime(&timeptr);

}


main(argc, argv)
  int argc;
  char *argv[];
{
  // Fast GoTo
  unsigned long fgoto_time = 0;
  int fgoto_mon = 0;
  int fgoto_day = 0;
  int fgoto_year = 0;

  // Eingabewert von getch() in der main-while-schleife
  int ieingabe;

  if ((argc == 2) ? ((argv[1][0] == '-') && (toupper(argv[1][1]) == 'V')) : 0) {
    allblackyintro(blacky_ver_info);

    printf("%s\n\n", blacky_ver_info);
    printkinfo(stdout);
    exit(0);
  }

  if ((argc > 1) ? ((argv[1][0] == '-') && ((argv[1][1] == '?') || (toupper(argv[1][1]) == 'H'))) : 0) {
    allblackyintro(blacky_ver_info);

    printf("Help:\n");
    printf("  bcal [-v | -p [days]]\n");
    printf("    -p: Print todays note (and follow days)\n");
    printf("    -v: Lizense, Version\n");
    printf("    -?: help: this screen\n");
    exit(0);
  }

  /* Write noticefiles to */
  blktxt = (char *) getenv("BLKTXT");
  if (blktxt == NULL) {
    char cd[255];

    sprintf(cd, "%s/txt", getenv("HOME"));
    blktxt = (char *) malloc(strlen(cd) + 1);
    if (blktxt == NULL)	{
      perror("malloc()");
      exit(1);
    }
    strcpy(blktxt, cd);
  }

  // Print NOTICE to STDOUT
  if ((argc > 1) ? ((argv[1][0] == '-') && ((argv[1][1] == 'p') || (toupper(argv[1][1]) == 'P'))) : 0) {
    unsigned long date;
    int i = 1;
    FILE *f;
    char cd[256];
    static struct tm *tminfo;

    time(&date);
    if (argc > 2) {
      i = atoi(argv[2]);
    }
    for (  ; i > 0; i--) {
      if ((f = fopen(makefilenamefromdate(date), "r")) != NULL) {
	tminfo = localtime(&date);
	printf("bcal: Notice found for %s d. %02d. %s(%d) %04d:\n", tage[tminfo -> tm_wday], tminfo -> tm_mday, mona[tminfo -> tm_mon], tminfo -> tm_mon + 1, tminfo -> tm_year + 1900);
	dosread(f, cd, 256);
	// Die Attribute einer Datei mssen nicht umbedingt ausgegeben werden
	if (cd[0] == '=') {
	  dosread(f, cd, 256);
	}
	while (!feof(f)) {
	  printf("> %s\n", cd);
	  dosread(f, cd, 256);
	}
	fclose(f);
      }
      date += SECOUNDADAY;
    }
    exit(0);
  }

  allblackyintro(blacky_ver_info);

  /* Editor? */
  EDITOR = (char *) getenv("EDITOR");
  if (EDITOR == NULL) {
    EDITOR = (char *) malloc(3);
    if (EDITOR == NULL)	{
      perror("malloc()");
      exit(1);
    }
    strcpy(EDITOR, "vi");
  }
  EDITOR2 = (char *) getenv("EDITOR2");
  if (EDITOR2 == NULL) {
    EDITOR2 = (char *) malloc(11);
    if (EDITOR2 == NULL)	{
      perror("malloc()");
      exit(1);
    }
    strcpy(EDITOR2, "xemacs -nw");
  }
  VIEWER = (char *) getenv("VIEWER");
  if (VIEWER == NULL) {
    VIEWER = (char *) malloc(5);
    if (VIEWER == NULL)	{
      perror("malloc()");
      exit(1);
    }
    strcpy(VIEWER, "less");
  }


  /* testwrite */
  {
    char cd[255];
    FILE *f;

    sprintf(cd, "%s/.testwrite", blktxt);
    if ((f = fopen(cd, "w+")) != NULL) {
      fclose(f);
      remove(cd);
    } else {
      char cd2[255];

      perror("fopen()");
      fprintf(stderr, "** WARNING: Can't write to %s\n", blktxt);
      sprintf(cd2, "mkdir -p %s", blktxt);
      fprintf(stderr, "Trying %s...\n", cd2);
      system(cd2);

      if ((f = fopen(cd, "w+")) != NULL) {
	fclose(f);
	remove(cd);
      } else {
	perror("fopen()");
	fprintf(stderr, "** ERROR: Can't create dir: %s\n", blktxt);
	fprintf(stderr, "** WARNING: Can't write to %s\n", blktxt);
      }

      fprintf(stderr, "Day-Notice-Files will be written to: %s\n", blktxt);
      fprintf(stderr, "You can change the directory with the enviroment\n" \
	      "variable BLKTXT.\n\nPress ENTER to continue...");

      getchar();

    }
  }


  // Using a Blacky2000 Timer File?
  {
    char *RET;
    if ((RET = (char *) getenv("Blacky2000TMRFile")) != NULL) {
      if ((Blacky2000TMR = (char *) malloc(strlen(RET) + 1)) != NULL) {
	strcpy(Blacky2000TMR, RET);
      }
    }
    if ((RET = (char *) getenv("Blacky2000TMRFile_HL")) != NULL) {
      if ((Blacky2000TMR_HL = (char *) malloc(strlen(RET) + 1)) != NULL) {
	strcpy(Blacky2000TMR_HL, RET);
      }
    }
  }

  // History init
  {
    int i;

    for (i = 0; i < histsize; i++) {
      hist[i] = 0L;
    }
  }
  
  init_cur_graph();

  // kein Cursor
  curs_set(0);

  /* Interrups vergeben ... */
  signal(SIGHUP, finish);       /* 01  /  hangup     - Alt-F4                            */
  signal(SIGINT, finish);       /* 02  /  Interrupt  - ^C                                */
  signal(SIGQUIT, finish);      /* 03  /  Quit                                           */
  signal(SIGTERM, finish);      /* 15  /  Terminierung                                   */
  /*     SIGALRM  */
  signal(SIGALRM, uhr);         /* 14  /  Alarm      - wenn gesetzte Zeit abgelaufen ist */
  /* Fenstergre gendert? */
  signal(SIGWINCH, winversch);  /* 28  /  ?          - grennderund des xTerms         */
  paint(0);
  uhr(128);
  /* ABSOLUTELY NO WARRANTY */
  gotoxy(3, 20);
  addstr("This is free software with ABSOLUTELY NO WARRANTY.");
  gotoxy(3, 21);
  addstr("For details read /home/andreas/.Blacky_NO_WARRANTY.");
  /* UHR */
  alarm(1);
  /* x-Term-Titel */
  blacky_xterm_set_titel("Blacky Calendar", NULL);
  /* Hauptschleife */
  do {
    gotoxy(14, 23);
    switch (ieingabe = getch()) {

      // Redraw  - C-l
    case ' ':
    case 12:
      endwin();
      paint(1);
      break;

      // <- (right) -- Woche zurck
    case 260:
    case 'h':
      if (curpos_w > 0) {
	if (caldis[curpos_d][curpos_w - 1] != 0) {
	  curpos_w--;
	} else {
	  if (curpos_w == 1) {
	    dateanz = (getaktdate(curpos_d, curpos_w) - (unsigned long) ((unsigned long) SECOUNDADAY * 7L));
	  } else {
	    if (curpos_w > 1) {
	      if ((caldis[curpos_d][curpos_w - 2] != 0)) {
		curpos_w -= 2;
	      }
	    }
	  }
	}
      } else {
	dateanz = (getaktdate(curpos_d, curpos_w) - (unsigned long) ((unsigned long) SECOUNDADAY * 7L));
      }
      paint(1);
      break;

      // -> (left) -- Woche vor
    case 261:
    case 'l':
      if (curpos_w < ((CALDIS_WEEKS + CALDIS_WEEKSSPACE) - 1)) {
	if (caldis[curpos_d][curpos_w + 1] != 0) {
	  curpos_w++;
	} else {
	  if (curpos_w == ((CALDIS_WEEKS + CALDIS_WEEKSSPACE) - 2)) {
	    dateanz = (getaktdate(curpos_d, curpos_w) + (unsigned long) ((unsigned long) SECOUNDADAY * 7L));
	  } else {
	    if (curpos_w < ((CALDIS_WEEKS + CALDIS_WEEKSSPACE) - 2)) {
	      if ((caldis[curpos_d][curpos_w + 2] != 0)) {
		curpos_w += 2;
	      }
	    }
	  }
	}
      } else {
	dateanz = (getaktdate(curpos_d, curpos_w) + (unsigned long) ((unsigned long) SECOUNDADAY * 7L));
      }
      paint(1);
      break;

      // up -- Tag zurck
    case 259:
    case 'k':
      if (curpos_d > 0) {
	if (caldis[curpos_d - 1][curpos_w] != 0) {
	  curpos_d--;
	} else {
	  if (curpos_w > 0) {
	    curpos_w--;
	    curpos_d--;
	  } else {
	    dateanz = (getaktdate(curpos_d, curpos_w) - (unsigned long) SECOUNDADAY);
	  }
	}
      } else {
	if (curpos_w > 0) {
	  if (caldis[6][curpos_w - 1] != 0) {
	    curpos_w--;
	    curpos_d = 6;
	  } else {
	    curpos_w -= 2;
	    curpos_d = 6;
	  }
	} else {
	  dateanz = (getaktdate(curpos_d, curpos_w) - (unsigned long) SECOUNDADAY);
	}
      }
      paint(1);
      break;

      // dn -- Tag vor
    case 258:
    case 'j':
      if (curpos_d < 6) {
	if (caldis[curpos_d + 1][curpos_w] != 0) {
	  curpos_d++;
	} else {
	  if (curpos_w < ((CALDIS_WEEKS + CALDIS_WEEKSSPACE) - 1)) {
	    curpos_d++;
	    curpos_w++;
	  } else {
	    dateanz = (getaktdate(curpos_d, curpos_w) + (unsigned long) SECOUNDADAY);
	  }
	}
      } else {
	if (curpos_w < ((CALDIS_WEEKS + CALDIS_WEEKSSPACE) - 1)) {
	  if (caldis[0][curpos_w + 1] != 0) {
	    curpos_d = 0;
	    curpos_w++;
	  } else {
	    curpos_d = 0;
	    curpos_w += 2;
	  }
	} else {
	  dateanz = (getaktdate(curpos_d, curpos_w) + (unsigned long) SECOUNDADAY);
	}
      }
      paint(1);
      break;

      // View Notice
    case 'v':
    case 'V':
    case '=':
      {
	char cd[500];
	FILE *f;

	alarm(0);
	endwin();
	sprintf(cd, "%s %s", VIEWER, makefilenamefromdate(getaktdate(curpos_d, curpos_w)));
	system(cd);

	calcattr(curpos_d, curpos_w);

	alarm(1);
	paint(1);
      }
      break;

      // Edit Notice
    case 13:
      {
	char cd[500];
	FILE *f;

	alarm(0);
	endwin();
	sprintf(cd, "%s %s", EDITOR, makefilenamefromdate(getaktdate(curpos_d, curpos_w)));
	system(cd);

	calcattr(curpos_d, curpos_w);

	alarm(1);
	paint(1);
      }
      break;
      
      // Edit Notice
    case '_':
      {
	char cd[500];
	FILE *f;

	alarm(0);
	endwin();
	sprintf(cd, "%s %s", EDITOR2, makefilenamefromdate(getaktdate(curpos_d, curpos_w)));
	system(cd);

	calcattr(curpos_d, curpos_w);

	alarm(1);
	paint(1);
      }
      break;


    case 'g':
    case 'G': {
      struct tm timeptr;
      int i;
      
      alarm(0);
      endwin();

      memset(&timeptr, 0, sizeof(timeptr));

      printf("\n\n\n\n\nGoTo Date:\n  year : ");
      fflush(stdout);
      scanf("%d", &i);
      // Angaben unter 50: 20XX
      if (i < 50) {
	i += 100;
      } else {
	// Angaben unter 100 ber 50: 19XX
	if (i < 100) {
	  // keine operationen ntig
	  // Alle Angaben: XXXX
	} else {
	  i -= 1900;
	}
      }
      timeptr.tm_year = i;

      printf("    01: Januar\t05: Mai  \t09: September\n");
      printf("    02: Februar\t06: Juni\t10: Oktober\n");
      printf("    03: Mearz\t07: Juli\t11: November\n");
      printf("    04: April\t08: August\t12: Dezember\n");

      printf("  month: ");
      fflush(stdout);
      scanf("%d", &i);
      if (i == 0) {
	goto NICHTOK;
      }
      timeptr.tm_mon  = i - 1;

      printf("  day  : ");
      fflush(stdout);
      scanf("%d", &i);
      if (i == 0) {
	goto NICHTOK;
      }
      timeptr.tm_mday = i;

      dateanz = (unsigned long) mktime(&timeptr);
      //      printf("%s\n", ctime(&dateanz));

    NICHTOK:
      paint(1);

      alarm(1);
    }
    break;

    // PG DN -- Monat vor
    case 338: {
      struct tm *timeptr;
      int i;
      long ti = getaktdate(curpos_d, curpos_w);

      
      //      memset(&timeptr, 0, sizeof(timeptr));
      timeptr = localtime(&ti);

      timeptr -> tm_mon++;
      dateanz = (unsigned long) mktime(timeptr);

      paint(1);
      alarm(1);
    }
    break;

    // PG UP -- Monat zurck
    case 339: {
      struct tm *timeptr;
      int i;
      long ti = getaktdate(curpos_d, curpos_w);

      
      //      memset(&timeptr, 0, sizeof(timeptr));
      timeptr = localtime(&ti);

      timeptr -> tm_mon--;
      dateanz = (unsigned long) mktime(timeptr);

      paint(1);
      alarm(1);
    }
    break;

    // HOME - Mo, 1. des Monats, 1. des Jahres
    case 262: {
      struct tm *timeptr;
      int i;
      long ti = getaktdate(curpos_d, curpos_w);

      
      //      memset(&timeptr, 0, sizeof(timeptr));
      timeptr = localtime(&ti);

      if ((timeptr -> tm_mon == 0) && (timeptr -> tm_mday == 1)) {
	break;
      }
      if (timeptr -> tm_mday == 1) {
	timeptr -> tm_mon = 0;
      } else {
	if (timeptr -> tm_wday == 1) {
	  timeptr -> tm_mday = 1;
	} else {
	  while (timeptr -> tm_wday != 1) {
	    ti -= SECOUNDADAY;
	    timeptr = localtime(&ti);
	  }
	}
      }

      dateanz = (unsigned long) mktime(timeptr);

      paint(1);
      alarm(1);
    }
    break;

    // END - So, letzter des Monats, letzter Tag des Jahres
    case 360: {
      struct tm *timeptr;
      int i;
      long ti = getaktdate(curpos_d, curpos_w);

      
      //      memset(&timeptr, 0, sizeof(timeptr));
      timeptr = localtime(&ti);

      if ((timeptr -> tm_mon == 11) && (timeptr -> tm_mday == mont[11])) {
	break;
      }
      if (timeptr -> tm_mday == mont[timeptr -> tm_mon]) {
	timeptr -> tm_mon = 11;
	timeptr -> tm_mday = mont[11];
      } else {
	if (timeptr -> tm_wday == 0) {
	  timeptr -> tm_mday = mont[timeptr -> tm_mon];
	} else {
	  while (timeptr -> tm_wday != 0) {
	    ti += SECOUNDADAY;
	    timeptr = localtime(&ti);
	  }
	}
      }

      dateanz = (unsigned long) mktime(timeptr);

      paint(1);
      alarm(1);
    }
    break;

    // Zeige Zeitdifferenz zu jetzt
    case 'c':
    case 'C':
    case 's':
    case 'S':

      {
	char *filename = (char *) makefilenamefromdate(getaktdate(curpos_d, curpos_w));
	FILE *f;
	int i, x;
	char cd[256];
	struct tm timeptr;
	unsigned long t_diff;
	double i2, ganz;
	// select()
	fd_set rfds;
	struct timeval tv;
	int retval;
	//
	int set_timepoint = 0;
	static unsigned long timepoint = 0;
	int show_ditt2timepoint = 1;

	if (timepoint == 0) {
	  show_ditt2timepoint = 0;
	}

	// set timpoint
	if ((ieingabe == 's') || (ieingabe == 'S')) {
	  set_timepoint = 1;
	}
	
	endwin();
	alarm(0);


	printf("\nc: years, months, weeks, days, hours, minutes, secounds\nC: years, weeks, days, hours, minutes, secounds\ns: set another timepoit to compare with\n\n\nTimePoints:\n");
	printf("    0: TP 12:00:00 Default TimePoint\n");
	// Suche TimePoints
	if ((f = fopen(filename, "r")) == NULL) {
	  perror(filename);
	  //	  getch();
	  //	  paint(1);
	  //	  break;
	} else {
	  for (i = 0; !feof(f); ) {
	    cd[0] = 0;
	    fgets(cd, 255, f);
	    if ((strncmp(cd, "TP ", 3) == 0) || (strncmp(cd, "DL ", 3) == 0)) {
	      i++;
	      printf("  %3d: %s", i, cd);
	      if (cd[strlen(cd) - 1] != '\n') {
		puts("");
	      }
	    }
	  }
	}
	if (i == 0) {
	  printf("\n\nWell, you can define timepoints in the datefile:\n");
	  printf("TP HH:MM:SS Comment\n");
	  printf("Exemple: TP 15:05:25 My Birthday\n\n");
	}
	printf("TimePoint: ");
	fflush(stdout);
	scanf("%d", &x);
	if (x > i) {
	  x = 0;
	}
	
	// Lade Datum in Zeit-Variable
	t_diff = getaktdate(curpos_d, curpos_w);
	memcpy((char *) &timeptr, (char *) localtime(&t_diff), sizeof(timeptr));

	timeptr.tm_sec = 0;
	timeptr.tm_min = 0;
	timeptr.tm_hour = 12;

	if (f != NULL) {
	  rewind(f);
	  if (x != 0) {
	    for (i = 1; (i <= x) && !feof(f); ) {
	      cd[0] = 0;
	      fgets(cd, 255, f);
	      if ((strncmp(cd, "TP ", 3) == 0) || (strncmp(cd, "DL ", 3) == 0)) {
		i++;
	      }
	    }

	    // sec
	    i = -1;
	    cd[11] = 0;
	    sscanf(cd + 9, "%d", &i);
	    if (i < 0 || i > 60) {
	      fprintf(stderr, "** Warning: Bad secods value: %d (-1>SS>61)\n", i);
	      i = 0;
	    }
	    timeptr.tm_sec = i;
	    // min
	    i = -1;
	    cd[8] = 0;
	    sscanf(cd + 6, "%d", &i);
	    if (i < 0 || i > 60) {
	      fprintf(stderr, "** Warning: Bad minutes value: %d (-1>MM>61)\n", i);
	      i = 0;
	    }
	    timeptr.tm_min = i;
	    // hour
	    i = -1;
	    cd[5] = 0;
	    sscanf(cd + 3, "%d", &i);
	    if (i < 0 || i > 24) {
	      fprintf(stderr, "** Warning: Bad hour value: %d (-1>HH>25)\n", i);
	      i = 0;
	    }
	    timeptr.tm_hour = i;
	  }
	}

	tp0 = (unsigned long) mktime(&timeptr);

	if (set_timepoint) {
	  timepoint = tp0;
	  break;
	}

	printf("TimePoint: %s", ctime(&tp0));
	printf("Press enter to continue...\n\n");

	if (show_ditt2timepoint == 1) {
	  printf("\nDIFF to %s", ctime(&timepoint));
	}

	  /*
((31*7+30*4+29)*1+(31*7+30*4+28)*3)/4   
365.25000
((31*7+30*4+29)*1+(31*7+30*4+28)*3)/4/12
30.43750
	  */


	if (ieingabe == 'c') {
	  while (1) {
	    if (show_ditt2timepoint == 1) {
	      timedatenow = timepoint;
	    } else {
	      time(&timedatenow);
	    }
	    
	    if (tp0 < timedatenow) {
	      printf("t = + ");
	      t_diff = timedatenow - tp0;
	    } else {
	      printf("t = - ");
	      t_diff = tp0 - timedatenow;
	    }
	    i2 = (double) t_diff;
	    //printf("%f -- %f\n", i2, difftime(timedatenow, tp0));

	    // years
	    modf(i2 / (60 * 60 * 24 * /*365.25*/ 365.2422), &ganz);
	    printf("%03d years  ", (int) ganz);
	    i2 = i2 - (ganz * (60 * 60 * 24 * /*365.25*/ 365.2422));

	    // (
	    // month
	    modf(i2 / (60 * 60 * 24 * 30.4375), &ganz);
	    printf("%02d month  ", (int) ganz);
	    i2 = i2 - (ganz * (60 * 60 * 24 * 30.4375));
	    // weeks
	    modf(i2 / (60 * 60 * 24 * 7), &ganz);
	    printf("%02d weeks  ", (int) ganz);
	    i2 = i2 - (ganz * (60 * 60 * 24 * 7));
	    // )

	    // days
	    modf(i2 / (60 * 60 * 24), &ganz);
	    printf("%02d days  ", (int) ganz);
	    i2 = i2 - (ganz * (60 * 60 * 24));

	    // hours
	    modf(i2 / (60 * 60), &ganz);
	    printf("%02d hours  ", (int) ganz);
	    i2 = i2 - (ganz * (60 * 60));

	    // minutes
	    modf(i2 / 60, &ganz);
	    printf("%02d min  ", (int) ganz);
	    i2 = i2 - (ganz * (60));

	    // sec
	    printf("%02d sec", (int) i2);
	  
	    fflush(stdout);
	    
	    //	  sleep(1);
	    FD_ZERO(&rfds);
	    FD_SET(0, &rfds);
	    tv.tv_sec = 1;
	    tv.tv_usec = 0;
	    if (!show_ditt2timepoint) {
	      retval = select(1, &rfds, NULL, NULL, &tv);
	      if (retval) {
		if (f != NULL) {
		  fclose(f);
		  f = NULL;
		}
		alarm(1);
		paint(1);
		break;	    
	      }
	    }
	   
	    if (show_ditt2timepoint == 1) {
	      show_ditt2timepoint = 0;
	      printf("\n\n\nDIFF to NOW\n");
	    }
	    for (i = (51 + 21 /* month & week */); i > 0; i--) {
	      printf("\b");
	    }
	  }
	} else {
	  while (1) {
	    if (show_ditt2timepoint == 1) {
	      timedatenow = timepoint;
	    } else {
	      time(&timedatenow);
	    }

	    if (tp0 < timedatenow) {
	      printf("t = + ");
	      t_diff = timedatenow - tp0;
	    } else {
	      printf("t = - ");
	      t_diff = tp0 - timedatenow;
	    }
	    i2 = (double) t_diff;
	    //printf("%f -- %f\n", i2, difftime(timedatenow, tp0));
	    
	    // years
	    modf(i2 / (60 * 60 * 24 * /*365.25*/ 365.2422), &ganz);
	    printf("%03d years  ", (int) ganz);
	    i2 = i2 - (ganz * (60 * 60 * 24 * /*365.25*/ 365.2422));
	    
	    // weeks
	    modf(i2 / (60 * 60 * 24 * 7), &ganz);
	    printf("%02d weeks  ", (int) ganz);
	    i2 = i2 - (ganz * (60 * 60 * 24 * 7));

	    // days
	    modf(i2 / (60 * 60 * 24), &ganz);
	    printf("%02d days  ", (int) ganz);
	    i2 = i2 - (ganz * (60 * 60 * 24));

	    // hours
	    modf(i2 / (60 * 60), &ganz);
	    printf("%02d hours  ", (int) ganz);
	    i2 = i2 - (ganz * (60 * 60));

	    // minutes
	    modf(i2 / 60, &ganz);
	    printf("%02d min  ", (int) ganz);
	    i2 = i2 - (ganz * (60));

	    // sec
	    printf("%02d sec", (int) i2);
	  
	    fflush(stdout);

	    //	  sleep(1);
	    FD_ZERO(&rfds);
	    FD_SET(0, &rfds);
	    tv.tv_sec = 1;
	    tv.tv_usec = 0;
	    if (!show_ditt2timepoint) {
	      retval = select(1, &rfds, NULL, NULL, &tv);
	      if (retval) {
		if (f != NULL) {
		  fclose(f);
		  f = NULL;
		}
		alarm(1);
		paint(1);
		break;	
	      }    
	    }
	    
	    if (show_ditt2timepoint == 1) {
	      show_ditt2timepoint = 0;
	      printf("\n\n\nDIFF to NOW\n");
	    }
	    for (i = (51 + 10/*+10 Week*/); i > 0; i--) {
	      printf("\b");
	    }
	  }
	}
	if (f != NULL) {
	  fclose(f);
	}
	alarm(1);
	paint(1);
      }
      break;

    // goto ToDay
    case 't':
    case 'T':
      time(&dateanz);
      paint(1);
      break;

    // print Help
    case 265:
    case '?':
    case 'H':
      painthelp(1);
      break;

      // schnell goto
    case '0':
    case '1':
    case '2':
    case '3':
    case '4':
    case '5':
    case '6':
    case '7':
    case '8':
    case '9': {
      struct tm timeptr;

      if ((timedatenow - fgoto_time) > 2) {
	fgoto_year = fgoto_mon = fgoto_day = 0;
      }
      fgoto_time = timedatenow;
      if (fgoto_year == 0) {
	if (fgoto_mon == 0) {
	  if ((fgoto_day == 0) ||
	      (fgoto_day > 3) ||
	      ((fgoto_day > 2) && ((ieingabe - 48) > 1)) ) {
	    fgoto_day = ieingabe - 48;
	  } else {
	    fgoto_day *= 10;
	    fgoto_day += ieingabe - 48;
	  }
	} else {
	  if ((fgoto_mon <= 0) ||
	      (fgoto_mon > 1) ||
	      ((fgoto_mon > 0) && (ieingabe - 48) > 2)) {
	    fgoto_mon = ieingabe - 48;
	  } else {
	    fgoto_mon *= 10;
	    fgoto_mon += ieingabe - 48;
	  }
	}
      } else {
	if (fgoto_year <= 0) {
	  fgoto_year = ieingabe - 48;
	} else {
	  fgoto_year *= 10;
	  fgoto_year += ieingabe - 48;
	}
      }
      if (fgoto_day > 0) {
	memset(&timeptr, 0, sizeof(timeptr));
	timeptr.tm_hour = (int) 12;
	if (fgoto_year > 0) {
          if (fgoto_year < 100) {
	    if (fgoto_year < 70) {
	      timeptr.tm_year = fgoto_year + 100;
	    } else {
	      timeptr.tm_year = fgoto_year;
	    }
	  } else {
	    timeptr.tm_year = fgoto_year - 1900;
	  }
	} else {
	  timeptr.tm_year = (int) caldis[9][curpos_w];
	}
	if (fgoto_mon > 0) {
	  timeptr.tm_mon  = fgoto_mon - 1;
	} else {
	  timeptr.tm_mon = (int) caldis[8][curpos_w] - 1;
	}
	timeptr.tm_mday = fgoto_day;
	dateanz = (unsigned long) mktime(&timeptr);
	//	fprintf(stderr, "%s - %d\n", ctime(&dateanz), fgoto_year);
	paint(1);
      }
    }
      break;

    case '.':
      if (fgoto_mon == 0) {
	fgoto_mon = -1;
      } else {
	if (fgoto_year == 0) {
	  fgoto_year = -1;
	}
      }
      break;
      
      // Search
    case '/':
      {
	char command[256], fname[256];
	if (srf != NULL) {
	  pclose(srf);
	}
	makeline(RABGE, HLINE, LABGE, 1, 15);
	makeline(VLINE, NOMBG, VLINE, 1, 16);
	makeline(VLINE, NOMBG, VLINE, 1, 17);
	makeline(VLINE, NOMBG, VLINE, 1, 18);
	makeline(VLINE, NOMBG, VLINE, 1, 19);
	makeline(VLINE, NOMBG, VLINE, 1, 20);
	makeline(VLINE, NOMBG, VLINE, 1, 21);
	makeline(VLINE, NOMBG, VLINE, 1, 22);
	makeline(VLINE, NOMBG, VLINE, 1, 23);
	makeline(LUECK, HLINE, RUECK, 1, 24);
	alarm(0);
	curs_set(1);
	gotoxy(3, 16);
	addstr("search:  ");
	editstr(searchstring, 15, 11, 0);
	gotoxy(3, 17);
	addstr("files:   ");
        strcpy(fname, blktxt);
	strcat(fname, "/bcal*txt");
	editstr(fname, 16, 11, 1);
	sprintf(command, "grep -il %s %s", searchstring, fname);
	gotoxy(3, 18);
	addstr("command: ");
	editstr(command, 17, 11, 1);
	alarm(0);
	endwin();
	printf("searching... (%s)\n\n\n", command);
	if ((srf = popen(command, "r")) == NULL) {
	  perror("popen(grep ..., r)");
	  sscanf("%s", fname);
	  paint(1);
	  alarm(1);
	  curs_set(0);
	  break;
	}
	curs_set(0);
	goto SEARCH_NEXT;
      }
      break;

    case 'n':
      {
      SEARCH_NEXT:
	if (srf != NULL) {
	  if (!feof(srf)) {
	    char cd[250];
	    char *cdx;
	    int i;
	    struct tm timeptr;

	    cd[0] = 0;
	    fgets(cd, 250, srf);
	    if (cd[0] != 0) {
	      cdx = (char *) strstr(cd, "bcal") + 4;
	      //	txt/bcal20010813.txt
	      alarm(0);
	      //	      endwin();

	      memset(&timeptr, 0, sizeof(timeptr));
	      // day
	      cdx[8] = 0;
	      sscanf(cdx + 6, "%d", &i);
	      timeptr.tm_mday = i;
	      if (i == 0) {
		goto SEARCH_RET_POINT;
	      }
	      // month
	      cdx[6] = 0;
	      sscanf(cdx + 4, "%d", &i);
	      timeptr.tm_mon  = i - 1;
	      if (i == 0) {
		goto SEARCH_RET_POINT;
	      }
	      // year
	      cdx[4] = 0;
	      sscanf(cdx, "%d", &i);
	      if (i == 0) {
		goto SEARCH_RET_POINT;
	      }
	      timeptr.tm_year = i - 1900;

	      dateanz = (unsigned long) mktime(&timeptr);
	    SEARCH_RET_POINT:
	      paint(1);
	      alarm(1);
	      break;
	    }
	  }
	}
      }
      paint(1);
      alarm(1);
      break;

      // ABN
      /*    case 'a':
    case 'A': {
      unsigned long date = getaktdate(curpos_d, curpos_w);
      static struct tm *timeptr, tmdata;
      char cd[100];
      int year, week, aby;

      timeptr = (struct tm *) localtime((const time_t *) &date);
      memcpy((char *) &tmdata, (char *) timeptr, sizeof(tmdata));
      timeptr = &tmdata;
      //      week = getweek(date);
      week = caldis[7][curpos_w];
      if (week == 0) {
	week = caldis[7][curpos_w - 1];
      }
      
      year = timeptr -> tm_year;
      // Ausbildungsjahr
      aby = year - 98;
      if (week >= 36) {
	aby++;
      }
      // Kalendaryear
      while (year >= 100) {
	year -= 100;
      }
      sprintf(cd, "w3m \'http://localhost/cgi-bin/abn-get.cgi?ausbyear=%1d&year=%02d&week=%02d&passwd=1234\'", aby, year, week);

      alarm(0);
      endwin();
      system(cd);
      paint(1);
      alarm(1);
    }
    break;
      */
      /* exit - 'ESC' || 'ALT-X' */
    case 27:
    case 'x':
    case 'X':
    case 'q':
    case 'Q':
      finish(0);
      break;

    case 'B': {
      if (inhist < histsize) {
	inhist++;
	dateanz = hist[inhist];
      }
      paint(1);
      break;
    }
      
      /* Wenn keine bekannte Taste gedrckt wurde */
    default:
      /*endwin();*/
      paint(1);
      break;


    case 9: // Wecheln der Darstellung
      linedisplay(0, getaktdate(curpos_d, curpos_w));
      break;

    case 'a': // Wecheln der Darstellung
      linedisplay(1, getaktdate(curpos_d, curpos_w));
      break;

    case 'A': // Wecheln der Darstellung
      linedisplay(1, timedatenow);
      break;


      //    case 'B':
    case 'b': {
      if (Blacky2000TMR != NULL) {

	alarm(0);
	endwin();

	printf("\n\nBlacky2000-Time-File: All timers active this Day\n\n\n");

	if ((int) run_timer(1, getaktdate(curpos_d, curpos_w)) != 1) {
	  printf("** ERROR: Blacky2000 Timer Faild!\n");
	}

	printf("\n\nPress enter to continue...\n");

	fflush(stdout);
	getch();

	paint(1);
	alarm(1);
	break;
      }
    }
    } /* switch() */     
   } while (1);

  /*
   GoToXY(50, 5);
   X := von Rechtz
   Y := von Links
  */

  /* move(o, l) :
      Bei NULL beginnend !!!
      o = von oben;
      l = von links; */
}


static void uhr(int sig)
{
  gettimedate();

  if (xdow != dow) {
    xdow = dow;
    sprintf(timedatenow_datestr, "%02s, %02d/%02d. %03s(%02d) %04d %03d/%02d", tage[dow], day, mont[mon - 1], mon3[mon - 1], mon, year, doy + 1, week);
    // Sicherstellen, das angezeigt wird...
    xdateanz = 0;
    endwin();
    paint(1);
    gotoxy(12, 4);
    addstr(timedatenow_datestr);
  }
  sprintf(timedatenow_timestr, "%02d:%02d:%02d", stu, min, sec);

  /* Ngster aufruf, nur von einem signal aufruf ! */
  if (sig != 128) {
    signal(SIGALRM, uhr);
    alarm(1);
  }
  /* Ausgeben */
  gotoxy(3, 4);
  addstr(timedatenow_timestr);
  //  gotoxy(12, 4);
  //  addstr(timedatenow_datestr);

  refresh();
}


static void winversch(int sig)
{
  signal(SIGWINCH, winversch);
  //  printf("%c", 7);
  endwin();
  paint(1);
}


static void finish(int sig)
{
  char cd[MULLLEN];

  endwin();
  if (srf != NULL) {
    pclose(srf);
  }

  /* x-Term-Titel */
  blacky_xterm_set_titel("xterm", 0L);
  exit(0);
}


/* ************************************************************************ */
/* Blacky2000 Timer */

/* Wenn G (Wert Gerade) oder U (Wert Ungerade) angegeben ist Prfen und
   1 zurckgeben wenn es hinkommt */
int test_gu(s, i)
  char *s;
  int i;
{
  if (s[1] != 0) {
    return 0;
  }
  if (toupper(s[0]) == 'G') {
    if ((i & 1) == 0) {
      return 1;
    }
  }
  if (toupper(s[0]) == 'U') {
    if ((i & 1) > 0) {
      return 1;
    }
  }
  return 0;
}


/* Trennt von - bis bedingungen auf. Aus "10-30" wird "10","30" */
zerlege_von_bis(s, i1, i2)
  char *s;
  int *i1, *i2;
{
  int i;

  for (i = 0; ((s[i] != '-') && (s[i] != 0)); i++)
    ;

  if (s[i] == 0) {
    *i1 = atoi(s);
    return;
  } else {
    s[i] = 0;
    *i1 = atoi(s);
    *i2 = atoi((s + (i + 1)));
    return;
  }
}

/* Trennt ein Feld in der Datei auf. z.B. aus "!10,5-56,G" wird
   "!10","5-56","G" */
trenne_felder(s, vara, varb, varc, vard, vare, varf, varg, varh, vari, varj)
  char *s, *vara, *varb, *varc, *vard, *vare, *varf, *varg, *varh, *vari, *varj;
{
  char *var[10];
  int i, ii, iii;

  var[0] = vara;
  var[1] = varb;
  var[2] = varc;
  var[3] = vard;
  var[4] = vare;
  var[5] = varf;
  var[6] = varg;
  var[7] = varh;
  var[8] = vari;
  var[9] = varj;

  ii = 0;
  for (i = 0; i < 10; i++) {
    for (iii = 0; ((s[ii] != 0) && (s[ii] != ',')); ii++) {
      var[i][iii] = s[ii];
      iii++;
    }
    if (s[ii] == ',') {
      ii++;
    }
    var[i][iii] = 0;
  }
}


/* Unterteil von run_timer. Dieser Abschnitt kmmert sich
   um jeweils eine Zeile. Wird entsprechend oft aufgerufen. 

   voll: 0: Kurtzausgabe (Command + Nachricht)
         1: Komplette rckgabe
*/
char *run_timer_line(cd, voll, ti)
  char *cd;
  int voll;
  long ti;
{
  char var[13][100], var2[10][10];
  int i, ii, iii;
  char B2KTMR_OK = 0;
  static char retline[255];

  int year, mon, day, dow, stu, min, doy, week;

  //  long ti = getaktdate(curpos_d, curpos_w);
  struct tm *tminfo;
  tminfo = localtime(&ti);
  year = (tminfo -> tm_year) + 1900; /* Jahr        */
  mon = (tminfo -> tm_mon) + 1;      /* Monat       */
  day = (tminfo -> tm_mday);         /* Tag         */
  dow = (tminfo -> tm_wday);         /* Wochentag   */
  stu = (tminfo -> tm_hour);         /* Stunde      */
  min = (tminfo -> tm_min);          /* Minute      */
  doy = (tminfo -> tm_yday);         /* Tag im Jahr */
  /* Woche */
  week = getweek(ti);



  /* Bei einem Leeren String return */
  if ((cd == NULL) ? 0 : (cd[0] == 0)) {
    return 0;
  }

  /* Fosichtshalber die Felder Initzialliesieren, falls welche Fehlern*/
  for (i = 0; i < 13; i++) {
    var[i][0] = '-';
    var[i][1] = 0;
  }

  /* Aufteilen der Zeile in die 12 Felder */
  sscanf(cd, "%s %s %s %s %s %s %s %s %s %s %s %s %s",
	 var[0],   /* Gruppe */
	 var[1],   /* DoW (Wochentag) */
	 var[2],   /* Jahr */
	 var[3],   /* Monat */
	 var[4],   /* Woche */
	 var[5],   /* Tag */
	 var[6],   /* Stunde */
	 var[7],   /* Minute */
	 var[8],   /* Aus */
	 var[9],   /* Kill */
	 var[10],  /* Disabel */
	 var[11],  /* Command */
	 var[12]); /* Text */
  /* Auswertung */
  /* Disabel */
  if (var[10][0] == '1') {
    return 0;
  }
  /* DoW */
  B2KTMR_OK = 0;;
  /* memset(var2, 0, 100); */
  trenne_felder(var[1], var2[0], var2[1], var2[2], var2[3], var2[4], var2[5], var2[6], var2[7], var2[8], var2[9]);
  /*printf("ts >%s<>%s<>%s<>%s<>%s<>%s<>%s<>%s<>%s<>%s<\n", var2[0], var2[1], var2[2], var2[3], var2[4], var2[5], var2[6], var2[7], var2[8], var2[9]);*/
  for (i = 0; i < 10; i++) {
    if (var2[i][0] != 0) {
      if (var2[i][0] == '-') {
	B2KTMR_OK = 1;
	break;
      } else {
	/* Normal */
	if (var2[i][0] != '!') {
	  if (strcasecmp(var2[i], tage[dow]) == 0) {
	    B2KTMR_OK = 1;
	    break;
	  }
	  /* Umgekehrt */
	} else {
	  if (strcasecmp((var2[i] + 1), tage[dow]) != 0) {
	    B2KTMR_OK = 1;
	    break;
	  }
	}
      }
    }
  }
  if (B2KTMR_OK == 0) {
    return 0;
  }

  /* Jahr */
  B2KTMR_OK = 0;;
  /* memset(var2, 0, 100); */
  trenne_felder(var[2], var2[0], var2[1], var2[2], var2[3], var2[4], var2[5], var2[6], var2[7], var2[8], var2[9]);
  for (i = 0; i < 10; i++) {
    if (var2[i][0] != 0) {
      if (var2[i][0] == '-') {
	B2KTMR_OK = 1;
	break;
      } else {
	/* Normal */
	if (var2[i][0] != '!') {
	  if (1 == test_gu(var2[i], year)) {
	    B2KTMR_OK = 1;
	    break;
	  }
	  ii = iii = 0;
	  zerlege_von_bis(var2[i], &ii, &iii);
	  /* Angabe eines Jahres */
	  if (iii == 0) {
	    if (((ii > 50) ? ((ii > 100) ? ii : 1900 + ii) : 2000 + ii) == year) {
	      B2KTMR_OK = 1;
	      break;
	    }
	    /* Angabe von bis */
	  } else {
	    if (year >= (((ii > 50) ? ((ii > 100) ? ii : 1900 + ii) : 2000 + ii)) &&
		(year <= ((iii > 50) ? ((iii > 100) ? iii : 1900 + iii) : 2000 + iii)))	{
	      B2KTMR_OK = 1;
	      break;
	    }
	  }
	  /* Umgekehrt */
	} else {
	  ii = iii = 0;
	  zerlege_von_bis((var2[i] + 1), &ii, &iii);
	  /* Angabe eines Jahres */
	  if (iii == 0) {
	    if (((ii > 50) ? ((ii > 100) ? ii : 1900 + ii) : 2000 + ii) != year) {
	      B2KTMR_OK = 1;
	      break;
	    }
	    /* Angabe von bis */
	  } else {
	    if (year < (((ii > 50) ? ((ii > 100) ? ii : 1900 + ii) : 2000 + ii)) ||
		(year > ((iii > 50) ? ((iii > 100) ? iii : 1900 + iii) : 2000 + iii))) {
	      B2KTMR_OK = 1;
	      break;
	    }
	  }
	}
      }
    }
  }
  if (B2KTMR_OK == 0) {
    return 0;
  }

  /* Monat */
  B2KTMR_OK = 0;;
  /* memset(var2, 0, 100); */
  trenne_felder(var[3], var2[0], var2[1], var2[2], var2[3], var2[4], var2[5], var2[6], var2[7], var2[8], var2[9]);
  for (i = 0; i < 10; i++) {
    if (var2[i][0] != 0) {
      if (var2[i][0] == '-') {
	B2KTMR_OK = 1;
	break;
      } else {
	/* Normal */
	if (var2[i][0] != '!') {
	  if (1 == test_gu(var2[i], mon)) {
	    B2KTMR_OK = 1;
	    break;
	  }
	  ii = iii = 0;
	  zerlege_von_bis(var2[i], &ii, &iii);
	  /* Angabe eines Monates */
	  if (iii == 0) {
	    if (ii == mon) {
	      B2KTMR_OK = 1;
	      break;
	    }
	    /* Angabe von bis */
	  } else {
	    if ((mon >= ii) &&
		(mon <= iii)) {
	      B2KTMR_OK = 1;
	      break;
	    }
	  }
	  /* Umgekehrt */
	} else {
	  ii = iii = 0;
	  zerlege_von_bis((var2[i] + 1), &ii, &iii);
	  /* Angabe eines Monates */
	  if (iii == 0) {
	    if (ii != mon) {
	      B2KTMR_OK = 1;
	      break;
	    }
	    /* Angabe von bis */
	  } else {
	    if ((mon < ii) ||
		(mon > iii)) {
	      B2KTMR_OK = 1;
	      break;
	    }
	  }
	}
      }
    }
  }
  if (B2KTMR_OK == 0) {
    return 0;
  }

  /* Woche */
  B2KTMR_OK = 0;;
  /* memset(var2, 0, 100); */
  trenne_felder(var[4], var2[0], var2[1], var2[2], var2[3], var2[4], var2[5], var2[6], var2[7], var2[8], var2[9]);
  for (i = 0; i < 10; i++) {
    if (var2[i][0] != 0) {
      if (var2[i][0] == '-') {
	B2KTMR_OK = 1;
	break;
      } else {
	/* Normal */
	if (var2[i][0] != '!') {
	  if (1 == test_gu(var2[i], week)) {
	    B2KTMR_OK = 1;
	    break;
	  }
	  ii = iii = -1;
	  zerlege_von_bis(var2[i], &ii, &iii);
	  /* Angabe einer Woche */
	  if (iii == -1) {
	    if (ii == week) {
	      B2KTMR_OK = 1;
	      break;
	    }
	    /* Angabe von bis */
	  } else {
	    if ((week >= ii) &&
		(week <= iii)) {
	      B2KTMR_OK = 1;
	      break;
	    }
	  }
	  /* Umgekehrt */
	} else {
	  ii = iii = -1;
	  zerlege_von_bis((var2[i] + 1), &ii, &iii);
	  /* Angabe eines Monates */
	  if (iii == -1) {
	    if (ii != week) {
	      B2KTMR_OK = 1;
	      break;
	    }
	    /* Angabe von bis */
	  } else {
	    if ((week < ii) ||
		(week > iii)) {
	      B2KTMR_OK = 1;
	      break;
	    }
	  }
	}
      }
    }
  }
  if (B2KTMR_OK == 0) {
    return 0;
  }
  /* Tag */
  B2KTMR_OK = 0;;
  /* memset(var2, 0, 100); */
  trenne_felder(var[5], var2[0], var2[1], var2[2], var2[3], var2[4], var2[5], var2[6], var2[7], var2[8], var2[9]);
  for (i = 0; i < 10; i++) {
    if (var2[i][0] != 0) {
      if (var2[i][0] == '-') {
	B2KTMR_OK = 1;
	break;
      } else {
	/* Normal */
	if (var2[i][0] != '!') {
	  if (1 == test_gu(var2[i], day)) {
	    B2KTMR_OK = 1;
	    break;
	  }
	  ii = iii = -1;
	  zerlege_von_bis(var2[i], &ii, &iii);
	  /* Angabe einer Woche */
	  if (iii == -1) {
	    if (ii == day) {
	      B2KTMR_OK = 1;
	      break;
	    }
	    /* Angabe von bis */
	  } else {
	    if ((day >= ii) &&
		(day <= iii)) {
	      B2KTMR_OK = 1;
	      break;
	    }
	  }
	  /* Umgekehrt */
	} else {
	  ii = iii = -1;
	  zerlege_von_bis((var2[i] + 1), &ii, &iii);
	  /* Angabe eines Tages */
	  if (iii == -1) {
	    if (ii != day) {
	      B2KTMR_OK = 1;
	      break;
	    }
	    /* Angabe von bis */
	  } else {
	    if ((day < ii) ||
		(day > iii)) {
	      B2KTMR_OK = 1;
	      break;
	    }
	  }
	}
      }
    }
  }
  if (B2KTMR_OK == 0) {
    return 0;
  }


  /*  if ((Blacky2000TMR_HL != NULL) && (btmrgrpok == -1)) {
    int i, l;
    char str[3];

    l = strlen(Blacky2000TMR_HL);
    btmrgrpok = 0;
    for (i = 0; i < l; i += 2) {
      strncpy(str, Blacky2000TMR_HL + i, 2);
      if (atoi(var[0]) == atoi(str)) {
		btmrgrpok = 1;
	break;
      }
    }
  }
  */
  if (voll == 1) {
    sprintf(retline, "%6s %10s %4s %2s %2s %2s %2s %2s %2s %2s %2s %15s %s\n",
	    var[0],   /* Gruppe */
	    var[1],   /* DoW (Wochentag) */
	    var[2],   /* Jahr */
	    var[3],   /* Monat */
	    var[4],   /* Woche */
	    var[5],   /* Tag */
	    var[6],   /* Stunde */
	    var[7],   /* Minute */
	    var[8],   /* Aus */
	    var[9],   /* Kill */
	    var[10],  /* Disabel */
	    var[11],  /* Command */
	    var[12]); /* Text */
    return(retline);
  }

  if (voll == 0) {
    sprintf(retline, "%s %s",
	    var[11],  /* Command */
	    var[12]); /* Text */
    return(retline);
  }

  return 0;
}





/* berprft die Timereintrge und fhrt sie aus,
   falls das ntig werden sollte

   voll: 0: Kurtzausgabe, rckgabe als Str
         1: Normale Ausgabe, auf stdout
*/
char *run_timer(voll, ti)
  int voll;
{
  FILE *f;
  int line; /* Zeile in Datei */
  int i;
  int CD_SIZE = 500;
  char cd[CD_SIZE];
  char *licd;
  static char ret[255];
  int retpointstr = 0;

  ret[0] = 0;

  if (Blacky2000TMR == NULL) {
    return 0;
  }

  if (voll == 1) {
    printf("GRUPPE        DoW Jahr Mo Wo Ta St Mi Au Ki Di         Comando Text\n");
  }

  if ((f = fopen(Blacky2000TMR, "r")) == NULL) {
    fprintf(stderr, "** FEHLER: Keine Timer-Datei!\n");
    perror(Blacky2000TMR);
    return 0;
  }
  i = 0;
  for (line = 1; !feof(f); line++) {
    dosread(f, cd, CD_SIZE);
    if ((cd[0] != '#') && (cd[0] != 0)) {
      licd = run_timer_line(cd, voll, ti);
      if (licd != NULL) {
	if (voll == 1) {
	  printf(licd);
	}
	if (voll == 0) {
	  strcpy(ret + retpointstr, licd);
	  strcat(ret, "; ");
	  retpointstr += strlen(licd) + 2;
	}
      }
    }
  }
  if (f != NULL) {
    fclose(f);
  }

  if (voll == 0) {
    return ret;
  }

  if (voll == 1) {
    return (char *) 1;
  }
}





/* Unterrotine zum Dahrstellen und Bedienen von BCAL im
   Line-Modus. Der Modus beginnt bei dem Datum <date_anfw>.

mode: 0: alle Tage anzeigen
      1: nur belegten Tage anzeigen
*/
int linedisplay(int mode, unsigned long date_anfw) {
  int exit = 0;
  line_cursor = 0;

  linefillbuffer(mode, date_anfw);
  paint(2);
  while (!exit) {
    gotoxy(14, 23);
    switch (getch()) {

      // Redraw  - C-l
    case ' ':
    case 12:
      endwin();
      paint(2);
      break;
      
      // Alle oder nur belegte
    case 'a':
    case 'A':
      if (mode == 0) {
	mode = 1;
      } else {
	mode = 0;
      }
      linefillbuffer(mode, linebgetaktdate(0));


      // up -- Tag zurck
    case 259:
    case 'k':
      if (line_cursor <= 0) {
	linefillbuffer(mode, linebgetaktdate(0) - SECOUNDADAY);
	//	line_cursor = LINESIZE - 1;
      } else {	
	line_cursor--;
      }

      paint(2);
      break;

      // dn -- Tag vor
    case 258:
    case 'j':
      if (linebuffer[linebgetakt(line_cursor) + 1].day != -1) {
	if (line_cursor >= LINESIZE - 1) {
	  linefillbuffer(mode, linebgetaktdate(0) + SECOUNDADAY);
	  //	linefillbuffer(mode, linebgetaktdate(LINESIZE - 1) + SECOUNDADAY);
	  //	line_cursor = 0;
	} else {
	  line_cursor++;
	}
      }

      paint(2);
      break;

      // View Notice
    case 'v':
    case 'V':
    case '=':
      {
	char cd[500];
	FILE *f;

	alarm(0);
	endwin();
	sprintf(cd, "%s %s", VIEWER, makefilenamefromdate(linebgetaktdate(line_cursor)));
	system(cd);

	linefillbuffer(mode, linebgetaktdate(0));

	alarm(1);
	paint(2);
      }
      break;

      // Edit Notice
    case 13:
      {
	char cd[500];
	FILE *f;

	alarm(0);
	endwin();
	sprintf(cd, "%s %s", EDITOR, makefilenamefromdate(linebgetaktdate(line_cursor)));
	system(cd);

	linefillbuffer(mode, linebgetaktdate(0));

	alarm(1);
	paint(2);
      }
      break;
      
      // Edit Notice
    case '_':
      {
	char cd[500];
	FILE *f;

	alarm(0);
	endwin();
	sprintf(cd, "%s %s", EDITOR2, makefilenamefromdate(linebgetaktdate(line_cursor)));
	system(cd);

	linefillbuffer(mode, linebgetaktdate(0));

	alarm(1);
	paint(2);
      }
      break;

    case 9: // Wecheln der Darstellung
      dateanz = (unsigned long) linebgetaktdate(line_cursor);
      paint(1);
      exit = 1;
      break;

    // goto ToDay
    case 't':
    case 'T':
      time(&dateanz);
      linefillbuffer(mode, dateanz);
      line_cursor = 0;
      paint(2);
      break;

      /* exit - 'ESC' || 'ALT-X' */
    case 27:
    case 'x':
    case 'X':
    case 'q':
    case 'Q':
      finish(0);
      break;

    } //switch (getch()) 
  }
}


/* Fllt den Datenpuffer <linebuffe:global>. Sucht nach
   ~/txt/bcalYYYYMMDD.txt und nach Timereintrge in ~/txt/timer.tmr
   und ld diese. Die Suche beginnt bei Datum <date_anf> und findet im
   Modus <mode> statt. */
linefillbuffer(int mode, unsigned long date_anf) {
  int MAX_VERSUCHT = 500;
  int versuche = 0;
  int i, i2;
  static struct tm *tminfo;
  unsigned long daydate_anf = date_anf;

  FILE *f;
  char line[64];


  for (i = 0; i < LINEBUFFERSIZE; i++) {
      linebuffer[i].day = -1;
    for (i2 = 0; i2 < LINEBUFFERMAXPERDAY; i2++) {
      linebuffer[i].note[i2] = NULL;
    }
  }


  for (i = 0; i < LINEBUFFERSIZE; i++) {
    versuche++;
    if (versuche > MAX_VERSUCHT) {
      break;
    }

    for (i2 = 0; i2 < LINEBUFFERMAXPERDAY; i2++) {
      if (linebuffer[i].note[i2] != NULL) {
	free(linebuffer[i].note[i2]);
      }
    }
    i2 = 0;

    linebuffer[i].week = getweek(daydate_anf);

    tminfo = localtime(&daydate_anf);
    linebuffer[i].attr = 0; // keine Hervorhebung

    linebuffer[i].dow = (tminfo -> tm_wday);
    linebuffer[i].day = (tminfo -> tm_mday);
    linebuffer[i].mon = (tminfo -> tm_mon + 1);
    linebuffer[i].year = (tminfo -> tm_year + 1900);
    
    // Aktueller Tag?
    if (year == (tminfo -> tm_year + 1900)) {
      if (mon == (tminfo -> tm_mon + 1)) {
	if (day == (tminfo -> tm_mday)) {
	  linebuffer[i].attr = 1;
	  //	  caldis[j][i] += 1 * (2 * 2 * 2 * 2 * 2); 
	}
      }
    }

    {
      char *tmpret = run_timer(0, daydate_anf);
      if (tmpret != NULL && tmpret[0] != 0) {
	int len = strlen(tmpret);

	linebuffer[i].note[i2] = (char *) malloc(len + 1 + 3);
	strcpy(linebuffer[i].note[i2], "TM ");
	strcat(linebuffer[i].note[i2++], tmpret);
      }
    }

    if ((f = fopen(makefilenamefromdate(linegetaktdate(i)), "r")) != NULL) {
      line[0] = 0;
      fgets(line, 63, f);
      if (linebuffer[i].attr == 0) {
	if (line[0] != 0) {
	  linebuffer[i].attr = 2;
	}

	if (line[0] == '=') {
	  Gross(line, line);
	  if (strncmp(line + 1, "GREEN", 5) == 0) {
	    linebuffer[i].attr = 3;
	  } else {
	    if (strncmp(line + 1, "RED", 3) == 0) {
	      linebuffer[i].attr = 4;
	    } else {
	      if (strncmp(line + 1, "MAGENTA", 7) == 0) {
		linebuffer[i].attr = 5;
	      } else {
		if (strncmp(line + 1, "BLUE", 4) == 0) {
		  linebuffer[i].attr = 6;
		} else {
		  if (strncmp(line + 1, "YELLOW", 6) == 0) {
		    linebuffer[i].attr = 7;
		  }
		}
	      }
	    }
	  }
	}
      }

      if (linebuffer[i].attr != 2) {
	fgets(line, 63, f);
      }
      
      if (line[0] != 0) {
	int len = strlen(line);
	if (line[len - 1] == 10) {
	  line[--len] = 0;
	}
	if ((strncmp(line, "TP ", 3) != 0) && (strncmp(line, "DL ", 3) != 0)) {
	  linebuffer[i].note[i2] = (char *) malloc(len + 1 + 3);
	  strcpy(linebuffer[i].note[i2], "   ");
	  strcat(linebuffer[i].note[i2++], line);
	} else {
	  linebuffer[i].note[i2] = (char *) malloc(len + 1);
	  strcpy(linebuffer[i].note[i2++], line);
	}
	while (!feof(f)) {
	  line[0] = 0;
	  fgets(line, 63, f);
	  if ((strncmp(line, "TP ", 3) == 0) || (strncmp(line, "DL ", 3) == 0)) {
	    int len = strlen(line);
	    if (line[len - 1] == 10) {
	      line[--len] = 0;
	    }
	    linebuffer[i].note[i2] = (char *) malloc(len + 1);
	    strcpy(linebuffer[i].note[i2++], line);
	  }
	  if (i2 >= LINEBUFFERMAXPERDAY) {
	    break;
	  }
	}
      } else {
	if ((mode != 0) && (i2 == 0)) {
	  i--;
	}
      }
      fclose(f);
    } else {
      if ((mode != 0) && (i2 == 0)) {
	i--;
      }
    }

    daydate_anf += SECOUNDADAY;
  }
}


/* Ordnet einer Bildschrimzeile <bline> eine Zeile im Datenpuffer
   <linebuffe:global> zu und gibt diese zurck */
int linebgetakt(int bline) {
  int i, j, i2;

  for (j = i = 0; j < LINESIZE; i++) {
    if (j == bline) {
      return i;
    }
    j++;
    for (i2 = 1; i2 < LINEBUFFERMAXPERDAY; i2++) {
      if (linebuffer[i].note[i2] != NULL) {
	if (j == bline) {
	  return i;
	}
	j++;
      }
    }
  }
}


/* Liefert das Datum welches der Bildschirmzeile <bline>
   entspricht. */
unsigned long linebgetaktdate(int bline) {
  return linegetaktdate(linebgetakt(bline));
}


/* Liefert das Datum welches der Zeile <line> im Datenpuffer
   <linebuffe:global> entspricht. */
unsigned long linegetaktdate(int line)
{
  static struct tm timeptr;

  memset(&timeptr, 0, sizeof(timeptr));

  timeptr.tm_mday = (int) (linebuffer[line].day);
  timeptr.tm_mon  = (int) (linebuffer[line].mon - 1);
  timeptr.tm_year = (int) (linebuffer[line].year - 1900);

  // Schaltsekunden: Von Zeit zu Zeit treten Schaltsekunden auf. Ist
  // die Zeit dann 0:00:00 wird nach 23:59:59 umgeschaltet, und das
  // ist der vorherige Tag. Dies frd zu Fehlern. Ist die Zeit
  // 12:00:00, ist die Schwelle weit entfernt...
  timeptr.tm_hour = (int) 12;

  return (unsigned long) mktime(&timeptr);
}
